#include <X11/Xos.h>
#include <stdio.h>
#include <X11/IntrinsicP.h>
#include <X11/StringDefs.h>
#include <X11/Xmu/Converters.h>
#include <math.h>

#include "BarGraphP.h"

static void BarGraphInitGCs
  (BarGraphWidget w)
{
  XtGCMask	valuemask;
  XGCValues	tmpGCV;
  
  valuemask = GCForeground | GCBackground;
  tmpGCV.background = w->core.background_pixel;
  
  tmpGCV.foreground = w->graph.foreground;
  w->graph.drawGC = XtGetGC ((Widget) w, valuemask, &tmpGCV);
  
  tmpGCV.foreground = w->core.background_pixel;
  w->graph.eraseGC = XtGetGC ((Widget) w, valuemask, &tmpGCV);
}

static void BarGraphInitialize
  (Widget req, Widget cre, ArgList args, Cardinal *num_args)
{
  BarGraphWidget w = (BarGraphWidget) cre;

  BarGraphInitGCs (w);
  w->graph.height = 0;
}

static void BarGraphResize
  (Widget gw)
{
  BarGraphWidget w = (BarGraphWidget) gw;

  if (! XtIsRealized (gw))
    return;

  w->graph.pix_height = (Dimension) (w->graph.height * w->core.height);
}
 
static void BarGraphRealize
  (Widget gw, XtValueMask *valueMask, XSetWindowAttributes *attrs)
{
  BarGraphWidget w = (BarGraphWidget) gw;
  
  XtCreateWindow (gw, InputOutput, CopyFromParent, *valueMask, attrs);
  BarGraphResize (gw);
}
  
static void BarGraphDestroy
  (Widget gw)
{
  BarGraphWidget w = (BarGraphWidget) gw;

  XtReleaseGC (gw, w->graph.drawGC);
  XtReleaseGC (gw, w->graph.eraseGC);
}

static void BarGraphRedisplay
  (Widget gw, XEvent *event, Region region)
{
    char sdisplay[10];
    
  BarGraphWidget w = (BarGraphWidget) gw;


  if (! XtIsRealized (gw)) { return; }

  XFillRectangle (XtDisplay (w), XtWindow (w),
		  w->graph.drawGC,
		  0, w->core.height - w->graph.pix_height,
		  w->core.width, w->graph.pix_height);
  
  XFillRectangle (XtDisplay (w), XtWindow (w),
		  w->graph.eraseGC,
		  0, 0,
		  w->core.width, w->core.height - w->graph.pix_height);
    sprintf(sdisplay, "%f\n", w->graph.height);
  XDrawString(XtDisplay (w), XtWindow (w), w->graph.drawGC, 4, 18, sdisplay, 10);
}

static void BarGraphChangePixelHeight
  (BarGraphWidget w, Dimension h)
{
  w->graph.pix_height = h;
  w->graph.height = ((double) w->graph.pix_height / (double) w->core.height);

  BarGraphRedisplay ((Widget) w, NULL, NULL);
}

static void BarGraphChangeHeight
  (BarGraphWidget w, double h)
{
  w->graph.height = h;
  w->graph.pix_height = (int) (w->core.height * h);

  BarGraphRedisplay ((Widget) w, NULL, NULL);
}

void BarGraphSetHeight
  (BarGraphWidget w, double h)
{
  if (h < 0) {
    BarGraphChangeHeight (w, 0);
  } else if (h > 1) {
    BarGraphChangeHeight (w, 1);
  } else {
    BarGraphChangeHeight (w, h);
  }
}

double BarGraphGetHeight
  (const BarGraphWidget w)
{
  return w->graph.height;
}

static void BarGraphSetPosition
  (Widget gw, XEvent* event, String* s, Cardinal* c)
{
  const BarGraphWidget w = (BarGraphWidget) gw;
  
  Position new_height =
    w->core.height - ((const XButtonEvent *) event)->y;

  if (new_height > w->core.height) {
    BarGraphChangePixelHeight (w, w->core.height);
  } else if (new_height < 0) {
    BarGraphChangePixelHeight (w, 0);
  } else {
    BarGraphChangePixelHeight (w, new_height);
  }
}
  
static void BarGraphMovePosition
  (Widget gw, XEvent* event, String* s, Cardinal* c)
{
  const BarGraphWidget w = (BarGraphWidget) gw;

  Position new_height =
    w->core.height - ((const XMotionEvent *) event)->y;

  if (new_height > w->core.height) {
    BarGraphChangePixelHeight (w, w->core.height);
  } else if (new_height < 0) {
    BarGraphChangePixelHeight (w, 0);
  } else {
    BarGraphChangePixelHeight (w, new_height);
  }
}

static void BarGraphNoOp
  (Widget gw, XEvent* event, String *s, Cardinal* c)
{
  return;
}

static void BarGraphMotionNotify
  (Widget gw, XEvent* event, String *s, Cardinal* c)
{
  const BarGraphWidget w = (BarGraphWidget) gw;

  XtCallCallbackList (gw, w->graph.motion_callbacks, NULL);
}

static void BarGraphPositionNotify
  (Widget gw, XEvent* event, String *s, Cardinal* c)
{
  const BarGraphWidget w = (BarGraphWidget) gw;

  XtCallCallbackList (gw, w->graph.position_callbacks, NULL);
}

static XtActionsRec bar_graph_actions[] =
{
  {"set-position",	BarGraphSetPosition},
  {"drag-start",	BarGraphNoOp},
  {"drag-move",		BarGraphMovePosition},
  {"drag-end",		BarGraphNoOp},
  {"motion-notify",	BarGraphMotionNotify},
  {"position-notify",	BarGraphPositionNotify},
};

#define offset(field) XtOffset (BarGraphWidget, field)

static XtResource bar_graph_resources[] = {

  {XtNwidth, XtCWidth, XtRDimension, sizeof (Dimension),
     offset (core.width), XtRImmediate, (XtPointer) 120},
  
  {XtNheight, XtCHeight, XtRDimension, sizeof (Dimension),
     offset (core.height), XtRImmediate, (XtPointer) 120},
  
  {XtNforeground, XtCForeground, XtRPixel, sizeof (Pixel),
     offset (graph.foreground), XtRString, XtDefaultForeground},

  {XtNplacementCallback, XtCCallback, XtRCallback, sizeof (XtPointer), 
     offset (graph.position_callbacks), XtRCallback, (XtPointer) NULL},
  
  {XtNmotionCallback, XtCCallback, XtRCallback, sizeof (XtPointer), 
     offset (graph.motion_callbacks), XtRCallback, (XtPointer) NULL},
};


#undef offset

BarGraphClassRec barGraphClassRec = {
  {
    /* core fields */
    
    /* superclass		*/	&widgetClassRec,
    /* class_name		*/	"BarGraph",
    /* size			*/	sizeof (BarGraphRec),
    /* class_initialize		*/	NULL,
    /* class_part_initialize	*/	NULL,
    /* class_inited		*/	FALSE,
    /* initialize		*/	BarGraphInitialize,
    /* initialize_hook		*/	NULL,
    /* realize			*/	BarGraphRealize,
    /* actions			*/	bar_graph_actions,
    /* num_actions		*/	XtNumber (bar_graph_actions),
    /* resources		*/	bar_graph_resources,
    /* num_resources		*/	XtNumber (bar_graph_resources),
    /* xrm_class		*/	NULL,
    /* compress_motion		*/	TRUE,
    /* compress_exposure	*/	TRUE,
    /* compress_enterleave	*/	TRUE,
    /* visible_interest		*/	FALSE,
    /* destroy			*/	BarGraphDestroy,
    /* resize			*/	BarGraphResize,
    /* expose			*/	BarGraphRedisplay,
    /* set_values		*/	NULL,
    /* set_values_hook		*/	NULL,
    /* set_values_almost	*/	NULL,
    /* get_values_hook		*/	NULL,
    /* accept_focus		*/	NULL,
    /* version			*/	XtVersion,
    /* callback_private		*/	NULL,
    /* tm_table			*/	NULL,
    /* query_geometry		*/	XtInheritQueryGeometry
    },
    {
      /* unused			*/	0,
    }
};

WidgetClass barGraphWidgetClass = (WidgetClass) &barGraphClassRec;
